@@ -2,41 +2,17 @@
     models/keycache.h
 
     This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2007 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2007 Klarälvdalens Datakonsult AB
 
-    Kleopatra is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Kleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_MODELS_KEYCACHE_H__
-#define __KLEOPATRA_MODELS_KEYCACHE_H__
+#pragma once
+
+#include "kleo_export.h"
 
 #include <QObject>
 
-#include <kleo_export.h>
-
 #include <gpgme++/global.h>
 
 #include <memory>
@@ -64,25 +40,45 @@ namespace Kleo
 {
 
 class FileSystemWatcher;
+class KeyGroup;
+class KeyGroupConfig;
+
+class KeyCacheAutoRefreshSuspension;
 
 class KLEO_EXPORT KeyCache : public QObject
 {
     Q_OBJECT
+
 protected:
     explicit KeyCache();
+
 public:
+    enum class KeyUsage {
+        AnyUsage,
+        Sign,
+        Encrypt,
+        Certify,
+        Authenticate,
+    };
+
     static std::shared_ptr<const KeyCache> instance();
     static std::shared_ptr<KeyCache> mutableInstance();
 
-    ~KeyCache();
+    ~KeyCache() override;
+
+    void setGroupsEnabled(bool enabled);
+    void setGroupConfig(const std::shared_ptr<KeyGroupConfig> &groupConfig);
 
     void insert(const GpgME::Key &key);
     void insert(const std::vector<GpgME::Key> &keys);
+    bool insert(const KeyGroup &group);
 
     void refresh(const std::vector<GpgME::Key> &keys);
+    bool update(const KeyGroup &group);
 
     void remove(const GpgME::Key &key);
     void remove(const std::vector<GpgME::Key> &keys);
+    bool remove(const KeyGroup &group);
 
     void addFileSystemWatcher(const std::shared_ptr<FileSystemWatcher> &watcher);
 
@@ -91,17 +87,28 @@ public:
     void setRefreshInterval(int hours);
     int refreshInterval() const;
 
+    std::shared_ptr<KeyCacheAutoRefreshSuspension> suspendAutoRefresh();
+
+    void enableRemarks(bool enable);
+    bool remarksEnabled() const;
+
     const std::vector<GpgME::Key> &keys() const;
     std::vector<GpgME::Key> secretKeys() const;
 
+    KeyGroup group(const QString &id) const;
+    std::vector<KeyGroup> groups() const;
+    std::vector<KeyGroup> configurableGroups() const;
+    void saveConfigurableGroups(const std::vector<KeyGroup> &groups);
+
     const GpgME::Key &findByFingerprint(const char *fpr) const;
     const GpgME::Key &findByFingerprint(const std::string &fpr) const;
 
+    std::vector<GpgME::Key> findByFingerprint(const std::vector<std::string> &fprs) const;
+
     std::vector<GpgME::Key> findByEMailAddress(const char *email) const;
     std::vector<GpgME::Key> findByEMailAddress(const std::string &email) const;
 
-    /** Look through the cache and search for the best key for a mailbox taking
-     * groups into account.
+    /** Look through the cache and search for the best key for a mailbox.
      *
      * The best key is the key with a UID for the provided mailbox that
      * has the highest validity and a subkey that is capable for the given
@@ -109,13 +116,27 @@ public:
      * If more then one key have a UID with the same validity
      * the most recently created key is taken.
      *
-     * The return value is a list because of groups. A group is
-     * a gnupg config entry that allows a user to configure multiple
-     * keys for a single mailbox. Only supported for OpenPGP currently.
+     * @returns the "best" key for the mailbox. */
+    GpgME::Key findBestByMailBox(const char *addr, GpgME::Protocol proto, KeyUsage usage) const;
+
+    /**
+     * Looks for a group named @a name which contains keys with protocol @a protocol
+     * that are suitable for the usage @a usage.
      *
-     * @returns only the "best" key for the mailbox, or a list if it is a group. */
-    std::vector<GpgME::Key> findBestByMailBox(const char *addr, GpgME::Protocol proto,
-                                              bool needSign, bool needEncrypt) const;
+     * If @a protocol is GpgME::OpenPGP or GpgME::CMS, then only groups consisting of keys
+     * matching this protocol are considered. Use @a protocol GpgME::UnknownProtocol to consider
+     * any groups regardless of the protocol including mixed-protocol groups.
+     *
+     * If @a usage is not KeyUsage::AnyUsage, then only groups consisting of keys supporting this usage
+     * are considered.
+     * The validity of keys and the presence of a private key (necessary for signing, certification, and
+     * authentication) is not taken into account.
+     *
+     * The first group that fulfills all conditions is returned.
+     *
+     * @returns a matching group or a null group if no matching group is found.
+     */
+    KeyGroup findGroup(const QString &name, GpgME::Protocol protocol, KeyUsage usage) const;
 
     const GpgME::Key &findByShortKeyID(const char *id) const;
     const GpgME::Key &findByShortKeyID(const std::string &id) const;
@@ -125,6 +146,9 @@ public:
 
     std::vector<GpgME::Key> findByKeyIDOrFingerprint(const std::vector<std::string> &ids) const;
 
+    const GpgME::Subkey &findSubkeyByKeyGrip(const char *grip, GpgME::Protocol protocol = GpgME::UnknownProtocol) const;
+    const GpgME::Subkey &findSubkeyByKeyGrip(const std::string &grip, GpgME::Protocol protocol = GpgME::UnknownProtocol) const;
+
     std::vector<GpgME::Subkey> findSubkeysByKeyID(const std::vector<std::string> &ids) const;
 
     std::vector<GpgME::Key> findRecipients(const GpgME::DecryptionResult &result) const;
@@ -139,19 +163,20 @@ public:
     std::vector<GpgME::Key> getGroupKeys(const QString &groupName) const;
 
     enum Option {
-        NoOption = 0,
+        // clang-format off
+        NoOption        = 0,
         RecursiveSearch = 1,
-        IncludeSubject = 2
+        IncludeSubject  = 2,
+        // clang-format on
     };
     Q_DECLARE_FLAGS(Options, Option)
 
     std::vector<GpgME::Key> findSubjects(const GpgME::Key &key, Options option = RecursiveSearch) const;
     std::vector<GpgME::Key> findSubjects(const std::vector<GpgME::Key> &keys, Options options = RecursiveSearch) const;
-    std::vector<GpgME::Key> findSubjects(std::vector<GpgME::Key>::const_iterator first, std::vector<GpgME::Key>::const_iterator last, Options options = RecursiveSearch) const;
+    std::vector<GpgME::Key>
+    findSubjects(std::vector<GpgME::Key>::const_iterator first, std::vector<GpgME::Key>::const_iterator last, Options options = RecursiveSearch) const;
 
     std::vector<GpgME::Key> findIssuers(const GpgME::Key &key, Options options = RecursiveSearch) const;
-    std::vector<GpgME::Key> findIssuers(const std::vector<GpgME::Key> &keys, Options options = RecursiveSearch) const;
-    std::vector<GpgME::Key> findIssuers(std::vector<GpgME::Key>::const_iterator first, std::vector<GpgME::Key>::const_iterator last, Options options = RecursiveSearch) const;
 
     /** Check if at least one keylisting was finished. */
     bool initialized() const;
@@ -159,6 +184,11 @@ public:
     /** Check if all keys have OpenPGP Protocol. */
     bool pgpOnly() const;
 
+    /** Set the keys the cache shall contain. Marks cache as initialized. Use for tests only. */
+    void setKeys(const std::vector<GpgME::Key> &keys);
+
+    void setGroups(const std::vector<KeyGroup> &groups);
+
 public Q_SLOTS:
     void clear();
     void startKeyListing(GpgME::Protocol proto = GpgME::UnknownProtocol)
@@ -169,11 +199,11 @@ public Q_SLOTS:
     void cancelKeyListing();
 
 Q_SIGNALS:
-    //void changed( const GpgME::Key & key );
-    void aboutToRemove(const GpgME::Key &key);
-    void added(const GpgME::Key &key);
     void keyListingDone(const GpgME::KeyListResult &result);
     void keysMayHaveChanged();
+    void groupAdded(const Kleo::KeyGroup &group);
+    void groupUpdated(const Kleo::KeyGroup &group);
+    void groupRemoved(const Kleo::KeyGroup &group);
 
 private:
     class RefreshKeysJob;
@@ -185,5 +215,3 @@ private:
 }
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(Kleo::KeyCache::Options)
-
-#endif /* __KLEOPATRA_MODELS_KEYCACHE_H__ */
