@@ -20,6 +20,7 @@
 
 #include <KLocalizedString>
 
+#include <QDate>
 #include <QIcon>
 #include <QVariant>
 
@@ -39,13 +40,17 @@ public:
         : mParentItem{parentItem}
         , mSig{sig}
     {
-        mItemData << QString::fromUtf8(sig.signerKeyID()) //
-                  << Formatting::prettyName(sig) //
-                  << Formatting::prettyEMail(sig) //
-                  << Formatting::creationDateString(sig) //
-                  << Formatting::expirationDateString(sig) //
-                  << Formatting::validityShort(sig) //
-                  << (sig.isExportable() ? QStringLiteral("✓") : QString());
+        const auto name = Formatting::prettyName(sig);
+        const auto email = Formatting::prettyEMail(sig);
+        mItemData = {
+            Formatting::prettyID(sig.signerKeyID()),
+            name,
+            email,
+            Formatting::creationDateString(sig),
+            Formatting::expirationDateString(sig),
+            Formatting::validityShort(sig),
+            sig.isExportable() ? QStringLiteral("✓") : QString{},
+        };
 
         QString lastNotation;
         if (showRemarks && parentItem) {
@@ -55,34 +60,58 @@ public:
                 }
             }
         }
-        mItemData << lastNotation;
+        mItemData.push_back(lastNotation);
 
-#ifdef GPGMEPP_SUPPORTS_TRUST_SIGNATURES
-        mItemData << Formatting::trustSignatureDomain(sig);
-#endif
+        const auto trustSignatureDomain = Formatting::trustSignatureDomain(sig);
+        mItemData.push_back(trustSignatureDomain);
+        mAccessibleText = {
+            Formatting::accessibleHexID(sig.signerKeyID()),
+            name.isEmpty() ? i18nc("text for screen readers for an empty name", "no name") : QVariant{},
+            email.isEmpty() ? i18nc("text for screen readers for an empty email address", "no email") : QVariant{},
+            Formatting::accessibleDate(Formatting::creationDate(sig)),
+            Formatting::accessibleExpirationDate(sig),
+            {}, // display text is always okay
+            sig.isExportable() ? i18nc("yes, is exportable", "yes") : i18nc("no, is not exportable", "no"),
+            lastNotation.isEmpty() ? i18nc("accessible text for empty list of tags", "none") : QVariant{},
+            trustSignatureDomain.isEmpty() ? i18n("not applicable") : QVariant{},
+        };
+        Q_ASSERT(mAccessibleText.size() == mItemData.size());
     }
 
     explicit UIDModelItem(const UserID &uid, UIDModelItem *parentItem)
         : mParentItem{parentItem}
         , mUid{uid}
     {
-        mItemData << Formatting::prettyUserID(uid);
+        mItemData = {Formatting::prettyUserID(uid)};
+        // for the empty cells of the user ID rows we announce "User ID"
+        mAccessibleText = {
+            {}, // use displayed user ID
+            i18n("User ID"),
+            i18n("User ID"),
+            i18n("User ID"),
+            i18n("User ID"),
+            i18n("User ID"),
+            i18n("User ID"),
+            i18n("User ID"),
+            i18n("User ID"),
+        };
     }
 
     // The root item
     UIDModelItem()
     {
-        mItemData << i18n("ID") //
-                  << i18n("Name") //
-                  << i18n("E-Mail") //
-                  << i18n("Valid From") //
-                  << i18n("Valid Until") //
-                  << i18n("Status") //
-                  << i18n("Exportable") //
-                  << i18n("Tags");
-#ifdef GPGMEPP_SUPPORTS_TRUST_SIGNATURES
-        mItemData << i18n("Trust Signature For");
-#endif
+        mItemData = {
+            i18n("User ID / Certification Key ID"),
+            i18n("Name"),
+            i18n("E-Mail"),
+            i18n("Valid From"),
+            i18n("Valid Until"),
+            i18n("Status"),
+            i18n("Exportable"),
+            i18n("Tags"),
+            i18n("Trust Signature For"),
+        };
+        // mAccessibleText is explicitly left empty
     }
 
     ~UIDModelItem()
@@ -126,6 +155,11 @@ public:
         return mItemData.value(column);
     }
 
+    QVariant accessibleText(int column) const
+    {
+        return mAccessibleText.value(column);
+    }
+
     QVariant toolTip(int column) const
     {
         if (!mSig.isNull()) {
@@ -172,6 +206,7 @@ public:
 private:
     QList<UIDModelItem *> mChildItems;
     QList<QVariant> mItemData;
+    QList<QVariant> mAccessibleText;
     UIDModelItem *mParentItem = nullptr;
     UserID::Signature mSig;
     UserID mUid;
@@ -268,6 +303,8 @@ QVariant UserIDListModel::headerData(int section, Qt::Orientation o, int role) c
     if (o == Qt::Horizontal && mRootItem) {
         if (role == Qt::DisplayRole || role == Qt::EditRole || role == Qt::ToolTipRole) {
             return mRootItem->data(section);
+        } else if (role == Qt::AccessibleTextRole) {
+            return mRootItem->accessibleText(section);
         }
     }
     return QVariant();
@@ -279,21 +316,22 @@ QVariant UserIDListModel::data(const QModelIndex &index, int role) const
         return QVariant();
     }
 
-    if (role != Qt::DisplayRole && role != Qt::EditRole && role != Qt::ToolTipRole && role != Qt::DecorationRole) {
-        return QVariant();
-    }
-
     auto item = static_cast<UIDModelItem *>(index.internalPointer());
 
-    if (role == Qt::ToolTipRole) {
+    switch (role) {
+    case Qt::DisplayRole:
+    case Qt::EditRole:
+        return item->data(index.column());
+    case Qt::AccessibleTextRole:
+        return item->accessibleText(index.column());
+    case Qt::ToolTipRole:
         return item->toolTip(index.column());
-    }
-
-    if (role == Qt::DecorationRole) {
+    case Qt::DecorationRole:
         return item->icon(index.column());
+    default:;
     }
 
-    return item->data(index.column());
+    return {};
 }
 
 UserID UserIDListModel::userID(const QModelIndex &index) const
@@ -305,9 +343,9 @@ UserID UserIDListModel::userID(const QModelIndex &index) const
     return item->uid();
 }
 
-QVector<UserID> UserIDListModel::userIDs(const QModelIndexList &indexes) const
+QList<UserID> UserIDListModel::userIDs(const QModelIndexList &indexes) const
 {
-    QVector<GpgME::UserID> ret;
+    QList<GpgME::UserID> ret;
     for (const QModelIndex &idx : indexes) {
         if (!idx.isValid()) {
             continue;
@@ -329,9 +367,9 @@ UserID::Signature UserIDListModel::signature(const QModelIndex &index) const
     return item->signature();
 }
 
-QVector<UserID::Signature> UserIDListModel::signatures(const QModelIndexList &indexes) const
+QList<UserID::Signature> UserIDListModel::signatures(const QModelIndexList &indexes) const
 {
-    QVector<GpgME::UserID::Signature> ret;
+    QList<GpgME::UserID::Signature> ret;
     for (const QModelIndex &idx : indexes) {
         if (!idx.isValid()) {
             continue;
@@ -348,3 +386,5 @@ void UserIDListModel::enableRemarks(bool value)
 {
     mRemarksEnabled = value;
 }
+
+#include "moc_useridlistmodel.cpp"
