@@ -23,6 +23,7 @@
 #include <libkleo/compliance.h>
 #include <libkleo/dn.h>
 #include <libkleo/formatting.h>
+#include <libkleo/keyhelpers.h>
 
 #include <KLocalizedString>
 #include <KMessageBox>
@@ -72,7 +73,7 @@ void Kleo::KeyRequester::init()
 
     if (DeVSCompliance::isCompliant()) {
         mComplianceIcon = new QLabel{this};
-        mComplianceIcon->setPixmap(QIcon::fromTheme(QStringLiteral("emblem-question")).pixmap(22));
+        mComplianceIcon->setPixmap(Formatting::questionIcon().pixmap(22));
     }
 
     // the label where the key id is to be displayed:
@@ -177,18 +178,11 @@ void Kleo::KeyRequester::setFingerprints(const QStringList &fingerprints)
     startKeyListJob(fingerprints);
 }
 
-static bool keyIsCompliant(const GpgME::Key &key)
-{
-    return (key.keyListMode() & GpgME::Validate) //
-        && Formatting::uidsHaveFullValidity(key) //
-        && Formatting::isKeyDeVs(key);
-}
-
 void Kleo::KeyRequester::updateKeys()
 {
     if (mKeys.empty()) {
         if (mComplianceIcon) {
-            mComplianceIcon->setPixmap(QIcon::fromTheme(QStringLiteral("emblem-unavailable")).pixmap(22));
+            mComplianceIcon->setPixmap(Formatting::unavailableIcon().pixmap(22));
             mComplianceIcon->setToolTip(QString{});
         }
         mLabel->clear();
@@ -219,11 +213,11 @@ void Kleo::KeyRequester::updateKeys()
         toolTipText += QLatin1Char('\n');
     }
     if (mComplianceIcon) {
-        if (Kleo::all_of(mKeys, &keyIsCompliant)) {
-            mComplianceIcon->setPixmap(QIcon::fromTheme(QStringLiteral("emblem-success")).pixmap(22));
+        if (Kleo::all_of(mKeys, &Kleo::DeVSCompliance::keyIsCompliant)) {
+            mComplianceIcon->setPixmap(Formatting::successIcon().pixmap(22));
             mComplianceIcon->setToolTip(DeVSCompliance::name(true));
         } else {
-            mComplianceIcon->setPixmap(QIcon::fromTheme(QStringLiteral("emblem-information")).pixmap(22));
+            mComplianceIcon->setPixmap(Formatting::warningIcon().pixmap(22));
             mComplianceIcon->setToolTip(DeVSCompliance::name(false));
         }
     }
@@ -240,7 +234,7 @@ static void showKeyListError(QWidget *parent, const GpgME::Error &err)
         "<qt><p>An error occurred while fetching "
         "the keys from the backend:</p>"
         "<p><b>%1</b></p></qt>",
-        QString::fromLocal8Bit(err.asString()));
+        Formatting::errorAsString(err));
 
     KMessageBox::error(parent, msg, i18n("Key Listing Failed"));
 }
@@ -517,3 +511,5 @@ void Kleo::SigningKeyRequester::virtual_hook(int id, void *data)
 {
     KeyRequester::virtual_hook(id, data);
 }
+
+#include "moc_keyrequester.cpp"
