@@ -37,7 +37,6 @@
 #include <QFile>
 #include <QPointer>
 #include <QProcess>
-#include <QRegExp>
 #include <QRegularExpression>
 #include <QStandardPaths>
 #include <QString>
@@ -136,156 +135,6 @@ QStringList Kleo::gnupgFolderWhitelist()
     };
 }
 
-namespace
-{
-class Gpg4win
-{
-public:
-    static const Gpg4win *instance()
-    {
-        // We use singleton to do the signature check only once.
-        static Gpg4win *inst = nullptr;
-        if (!inst) {
-            inst = new Gpg4win();
-        }
-        return inst;
-    }
-
-private:
-    QString mVersion;
-    QString mDescription;
-    QString mDescLong;
-    QString mBrandingWindowTitle;
-    QString mBrandingIcon;
-    bool mSignedVersion;
-
-    Gpg4win()
-        : mVersion(QStringLiteral("Unknown Windows Version"))
-        , mDescription(i18n("Certificate Manager and Unified Crypto GUI"))
-        , mDescLong(QStringLiteral("<a href=https://www.gpg4win.org>Visit the Gpg4win homepage</a>"))
-        , mSignedVersion(false)
-    {
-        const QString instPath = Kleo::gpg4winInstallPath();
-        const QString verPath = instPath + QStringLiteral("/../VERSION");
-        QFile versionFile(verPath);
-
-        // Open the file first to avoid a verify and then read issue where
-        // "auditors" might say its an issue,...
-        if (!versionFile.open(QIODevice::ReadOnly)) {
-            return;
-        }
-        // Expect a three line format of three HTML strings.
-        const auto versVersion = QString::fromUtf8(versionFile.readLine()).trimmed();
-        const auto versDescription = QString::fromUtf8(versionFile.readLine()).trimmed();
-        const auto versDescLong = QString::fromUtf8(versionFile.readLine()).trimmed();
-        // read optional two branding strings
-        const auto brandingWindowTitle = QString::fromUtf8(versionFile.readLine()).trimmed();
-        const auto brandingIcon = QString::fromUtf8(versionFile.readLine()).trimmed();
-
-        const QString sigPath = verPath + QStringLiteral(".sig");
-        QFileInfo versionSig(instPath + QStringLiteral("/../VERSION.sig"));
-        if (versionSig.exists()) {
-            /* We have a signed version so let us check it against the GnuPG
-             * release keys. */
-            QProcess gpgv;
-            gpgv.setProgram(Kleo::gpgPath().replace(QStringLiteral("gpg.exe"), QStringLiteral("gpgv.exe")));
-            const QString keyringPath(QStringLiteral("%1/../share/gnupg/distsigkey.gpg").arg(Kleo::gnupgInstallPath()));
-            gpgv.setArguments(QStringList() << QStringLiteral("--keyring") << keyringPath << QStringLiteral("--") << sigPath << verPath);
-            gpgv.start();
-            gpgv.waitForFinished();
-            if (gpgv.exitStatus() == QProcess::NormalExit && !gpgv.exitCode()) {
-                qCDebug(LIBKLEO_LOG) << "Valid Version: " << versVersion;
-                mVersion = versVersion;
-                mDescription = versDescription;
-                mDescLong = versDescLong;
-                mBrandingWindowTitle = brandingWindowTitle;
-                mBrandingIcon = brandingIcon;
-                mSignedVersion = true;
-            } else {
-                qCDebug(LIBKLEO_LOG) << "gpgv failed with stderr: " << gpgv.readAllStandardError();
-                qCDebug(LIBKLEO_LOG) << "gpgv stdout" << gpgv.readAllStandardOutput();
-            }
-        } else {
-            qCDebug(LIBKLEO_LOG) << "No signed VERSION file found.";
-        }
-        // Also take Version information from unsigned Versions.
-        mVersion = versVersion;
-    }
-
-public:
-    const QString &version() const
-    {
-        return mVersion;
-    }
-    const QString &description() const
-    {
-        return mDescription;
-    }
-    const QString &longDescription() const
-    {
-        return mDescLong;
-    }
-    bool isSignedVersion() const
-    {
-        return mSignedVersion;
-    }
-    const QString &brandingWindowTitle() const
-    {
-        return mBrandingWindowTitle;
-    }
-    const QString &brandingIcon() const
-    {
-        return mBrandingIcon;
-    }
-};
-} // namespace
-
-bool Kleo::gpg4winSignedversion()
-{
-    return Gpg4win::instance()->isSignedVersion();
-}
-
-QString Kleo::gpg4winVersionNumber()
-{
-    // extract the actual version number from the string returned by Gpg4win::version();
-    // we assume that Gpg4win::version() returns a version number (conforming to the semantic
-    // versioning spec) optionally prefixed with some text followed by a dash,
-    // e.g. "Gpg4win-3.1.15-beta15"; see https://dev.gnupg.org/T5663
-    static const QRegularExpression catchSemVerRegExp{QLatin1String{R"(-([0-9]+(?:\.[0-9]+)*(?:-[.0-9A-Za-z-]+)?(?:\+[.0-9a-zA-Z-]+)?)$)"}};
-
-    QString ret;
-    const auto match = catchSemVerRegExp.match(gpg4winVersion());
-    if (match.hasMatch()) {
-        ret = match.captured(1);
-    } else {
-        ret = gpg4winVersion();
-    }
-    qCDebug(LIBKLEO_LOG) << __func__ << "returns" << ret;
-    return ret;
-}
-
-QString Kleo::gpg4winVersion()
-{
-    return Gpg4win::instance()->version();
-}
-QString Kleo::gpg4winDescription()
-{
-    return Gpg4win::instance()->description();
-}
-QString Kleo::gpg4winLongDescription()
-{
-    return Gpg4win::instance()->longDescription();
-}
-
-QString Kleo::brandingWindowTitle()
-{
-    return Gpg4win::instance()->brandingWindowTitle();
-}
-QString Kleo::brandingIcon()
-{
-    return Gpg4win::instance()->brandingIcon();
-}
-
 QString Kleo::gpg4winInstallPath()
 {
 #ifdef Q_OS_WIN
@@ -376,9 +225,11 @@ static std::array<int, 3> getVersionFromString(const char *actual, bool &ok)
     QString versionString = QString::fromLatin1(actual);
 
     // Try to fix it up
-    QRegExp rx(QLatin1String(R"((\d+)\.(\d+)\.(\d+)(?:-svn\d+)?.*)"));
+    QRegularExpression rx(QRegularExpression::anchoredPattern(QLatin1String(R"((\d+)\.(\d+)\.(\d+)(?:-svn\d+)?.*)")));
+    QRegularExpressionMatch match;
     for (int i = 0; i < 3; i++) {
-        if (!rx.exactMatch(versionString)) {
+        match = rx.match(versionString);
+        if (!match.hasMatch()) {
             versionString += QStringLiteral(".0");
         } else {
             ok = true;
@@ -392,7 +243,7 @@ static std::array<int, 3> getVersionFromString(const char *actual, bool &ok)
     }
 
     for (int i = 0; i < 3; ++i) {
-        ret[i] = rx.cap(i + 1).toUInt(&ok);
+        ret[i] = match.capturedView(i + 1).toUInt(&ok);
         if (!ok) {
             return ret;
         }
@@ -501,31 +352,26 @@ bool Kleo::gnupgIsDeVsCompliant()
 }
 
 #ifdef Q_OS_WIN
-static unsigned int gpgConfGetConsoleOutputCodePage()
+static unsigned int guessConsoleOutputCodePage()
 {
-    // calls `gpgconf --show-codepages` to determine the console output codepage used by gpg on Windows;
-    // for other OSs gpgconf returns nothing
-    const auto gpgConfPath = Kleo::gpgConfPath();
-    if (gpgConfPath.isEmpty()) {
-        return 0;
+    /* Qt on Windows uses GetACP while GnuPG prefers
+     * GetConsoleOutputCP.
+     *
+     * As we are not a console application GetConsoleOutputCP
+     * usually returns 0.
+     * From experience the closest thing that let's us guess
+     * what GetConsoleOutputCP returns for a console application
+     * it appears to be the OEMCP.
+     */
+    unsigned int cpno = GetConsoleOutputCP();
+    if (!cpno) {
+        cpno = GetOEMCP();
     }
-    QProcess gpgConf;
-    qCDebug(LIBKLEO_LOG) << __func__ << "starting" << gpgConfPath << "--show-codepages";
-    gpgConf.start(gpgConfPath, {QStringLiteral("--show-codepages")});
-    if (!gpgConf.waitForFinished()) {
-        qCDebug(LIBKLEO_LOG) << __func__ << "failed to execute gpgconf:" << gpgConf.errorString() << "\nstderr:" << gpgConf.readAllStandardError();
-        return 0;
+    if (!cpno) {
+        cpno = GetACP();
     }
-
-    unsigned int cpno = 0;
-    const auto lines = gpgConf.readAllStandardOutput().split('\n');
-    // look for a line of the form "Console: CP%u" or "Console: CP%u/CP%u"
-    for (const auto &l : lines) {
-        if (l.startsWith("Console: CP")) {
-            // the console output codepage is the second CP value if there are two
-            cpno = l.mid(l.lastIndexOf("CP") + 2).toUInt();
-            break;
-        }
+    if (!cpno) {
+        qCDebug(LIBKLEO_LOG) << __func__ << "Failed to find native codepage";
     }
 
     qCDebug(LIBKLEO_LOG) << __func__ << "returns" << cpno;
@@ -534,39 +380,60 @@ static unsigned int gpgConfGetConsoleOutputCodePage()
 
 static QString fromEncoding(unsigned int src_encoding, const char *data)
 {
+    if (!data || !*data) {
+        return {};
+    }
+
+    // returns necessary buffer size including the terminating null character
     int n = MultiByteToWideChar(src_encoding, 0, data, -1, NULL, 0);
-    if (n < 0) {
+    if (n <= 0) {
+        qCDebug(LIBKLEO_LOG) << __func__ << "determining necessary buffer size failed with error code" << GetLastError();
         return QString();
     }
 
     wchar_t *result = (wchar_t *)malloc((n + 1) * sizeof *result);
 
     n = MultiByteToWideChar(src_encoding, 0, data, -1, result, n);
-    if (n < 0) {
+    if (n <= 0) {
         free(result);
+        qCDebug(LIBKLEO_LOG) << __func__ << "conversion failed with error code" << GetLastError();
         return QString();
     }
-    const auto ret = QString::fromWCharArray(result, n);
+    const auto ret = QString::fromWCharArray(result, n - 1);
     free(result);
     return ret;
 }
+
+static QString stringFromGpgOutput_legacy(const QByteArray &ba)
+{
+    static const unsigned int cpno = guessConsoleOutputCodePage();
+
+    if (cpno) {
+        qCDebug(LIBKLEO_LOG) << __func__ << "trying to decode" << ba << "using codepage" << cpno;
+        const auto rawData = QByteArray{ba}.replace("\r\n", "\n");
+        const auto s = fromEncoding(cpno, rawData.constData());
+        if (!s.isEmpty() || ba.isEmpty()) {
+            return s;
+        }
+        qCDebug(LIBKLEO_LOG) << __func__ << "decoding output failed; falling back to QString::fromLocal8Bit()";
+    }
+    qCDebug(LIBKLEO_LOG) << __func__ << "decoding from local encoding:" << ba;
+    return QString::fromLocal8Bit(ba);
+}
 #endif
 
 QString Kleo::stringFromGpgOutput(const QByteArray &ba)
 {
 #ifdef Q_OS_WIN
-    static const unsigned int cpno = gpgConfGetConsoleOutputCodePage();
-
-    if (cpno) {
-        qCDebug(LIBKLEO_LOG) << __func__ << "trying to decode" << ba << "using codepage" << cpno;
-        const auto s = fromEncoding(cpno, ba.constData());
-        if (!s.isEmpty() || ba.isEmpty()) {
-            return s;
-        }
-        qCDebug(LIBKLEO_LOG) << __func__ << "decoding output failed; falling back to QString::fromLocal8Bit()";
+    // since 2.2.28, GnuPG always uses UTF-8 for console output (and input)
+    if (Kleo::engineIsVersion(2, 2, 28, GpgME::GpgEngine)) {
+        return QString::fromUtf8(ba);
+    } else {
+        return stringFromGpgOutput_legacy(ba);
     }
-#endif
+#else
     return QString::fromLocal8Bit(ba);
+#endif
 }
 
 QStringList Kleo::backendVersionInfo()
@@ -703,3 +570,92 @@ void Kleo::killDaemons()
 
     process = startGpgConf({QStringLiteral("--kill"), QStringLiteral("all")});
 }
+
+const std::vector<std::string> &Kleo::availableAlgorithms()
+{
+    static const std::vector<std::string> algos = {
+        "brainpoolP256r1",
+        "brainpoolP384r1",
+        "brainpoolP512r1",
+        "curve25519",
+        "curve448",
+        "nistp256",
+        "nistp384",
+        "nistp521",
+        "rsa2048",
+        "rsa3072",
+        "rsa4096",
+        // "secp256k1", // Curve secp256k1 is explicitly ignored
+    };
+    return algos;
+}
+
+const std::vector<std::string> &Kleo::preferredAlgorithms()
+{
+    static const std::vector<std::string> algos = {
+        "curve25519",
+        "brainpoolP256r1",
+        "rsa3072",
+        "rsa2048",
+    };
+    return algos;
+}
+
+const std::vector<std::string> &Kleo::ignoredAlgorithms()
+{
+    static const std::vector<std::string> algos = {
+        "secp256k1", // Curve secp256k1 is not useful
+    };
+    return algos;
+}
+
+bool Kleo::gpgvVerify(const QString &filePath, const QString &sigPath, const QString &keyring, const QStringList &additionalSearchPaths)
+{
+    const QFileInfo verifyFi(filePath);
+    if (!verifyFi.isReadable()) {
+        return false;
+    } else {
+        qCDebug(LIBKLEO_LOG) << "Verifying" << filePath;
+    }
+
+    const auto gpgvPath = QStandardPaths::findExecutable(QStringLiteral("gpgv"), additionalSearchPaths);
+    if (gpgvPath.isEmpty()) {
+        qCDebug(LIBKLEO_LOG) << "Could not find gpgv";
+        return false;
+    }
+
+    QFileInfo sigFi;
+    if (!sigPath.isEmpty()) {
+        sigFi.setFile(sigPath);
+    } else {
+        sigFi.setFile(filePath + QStringLiteral(".sig"));
+    }
+
+    if (!sigFi.isReadable()) {
+        qCDebug(LIBKLEO_LOG) << "No signature found at" << sigFi.absoluteFilePath();
+        return false;
+    }
+
+    auto process = QProcess();
+    process.setProgram(gpgvPath);
+    QStringList args;
+    if (!keyring.isEmpty()) {
+        args << QStringLiteral("--keyring") << keyring;
+    }
+    args << QStringLiteral("--") << sigFi.absoluteFilePath() << verifyFi.absoluteFilePath();
+    process.setArguments(args);
+    qCDebug(LIBKLEO_LOG).nospace() << "Starting gpgv (" << gpgvPath << ") with arguments " << args.join(QLatin1Char(' ')) << " ...";
+    process.start();
+
+    if (!process.waitForFinished(-1)) {
+        qCDebug(LIBKLEO_LOG) << "Failed to execute gpgv" << process.errorString();
+    }
+    bool ret = (process.exitStatus() == QProcess::NormalExit && process.exitCode() == 0);
+
+    if (!ret) {
+        qCDebug(LIBKLEO_LOG) << "Failed to verify file";
+        qCDebug(LIBKLEO_LOG) << "gpgv stdout:" << QString::fromUtf8(process.readAllStandardOutput());
+        qCDebug(LIBKLEO_LOG) << "gpgv stderr:" << QString::fromUtf8(process.readAllStandardError());
+    }
+    return ret;
+}
